@@ -183,10 +183,12 @@ class LoadMaster(HttpClient):
 
     def build_virtual_service(self, service):
         """Create a VirtualService instance with standard defaults"""
+        is_sub_vs = True if int(service.get('MasterVSID', 0)) != 0 else False
         virt_serv = VirtualService(self.access_info,
                                    service.get('VSAddress'),
                                    service.get('VSPort'),
-                                   service.get('Protocol'))
+                                   service.get('Protocol'),
+                                   is_sub_vs=is_sub_vs)
         virt_serv.populate_default_attributes(service)
         return virt_serv
 
@@ -366,7 +368,7 @@ class KempBaseObjectModel(object):
     )
 
     def __repr__(self):
-        return '{} {}'.format(self.__class__.__name__, self.to_api_dict())
+        return '{} {}'.format(self.__class__.__name__, self.to_dict())
 
     def to_api_dict(self):
         """returns API related attributes as a dict
@@ -381,15 +383,29 @@ class KempBaseObjectModel(object):
             api_dict[key] = value
         return api_dict
 
+    def to_dict(self):
+        """returns attributes whose values are not None or whose name starts
+        with _ as a dict"""
+        api_dict = {}
+        for key, value in self.__dict__.items():
+            if key.startswith("_") or value is None:
+                continue
+            api_dict[key] = value
+        return api_dict
+
 
 class VirtualService(HttpClient, KempBaseObjectModel):
 
-    def __init__(self, loadmaster_info, ip, port=80, prot="tcp"):
-        self.vs = ip
-        validate_ip(ip)
+    def __init__(self, loadmaster_info, ip, port=80, prot="tcp",
+                 is_sub_vs=False):
+        self.index = None  # to avoid AttributeErrors later
+        if not is_sub_vs:
+            # Skip validation when it is a subvs as they do not have ip/port
+            self.vs = ip
+            validate_ip(ip)
 
-        self.port = port
-        validate_port(port)
+            self.port = port
+            validate_port(port)
 
         self.prot = prot
         validate_protocol(prot)
@@ -410,17 +426,27 @@ class VirtualService(HttpClient, KempBaseObjectModel):
 
     def _get_base_parameters(self):
         """Returns the bare minimum VS parameters. IP, port and protocol"""
-        return {"vs": self.vs, "port": self.port, "prot": self.prot}
+        if self.index is None:
+            return {"vs": self.vs, "port": self.port, "prot": self.prot}
+        else:
+            return {"vs": self.index}
 
     @property
     def access_info(self):
-        return {
-            "endpoint": self.endpoint,
-            "ip_address": self.ip_address,
-            "vs": self.vs,
-            "port": self.port,
-            "prot": self.prot,
-        }
+        if self.index is None:
+            return {
+                "endpoint": self.endpoint,
+                "ip_address": self.ip_address,
+                "vs": self.vs,
+                "port": self.port,
+                "prot": self.prot,
+            }
+        else:
+            return {
+                "endpoint": self.endpoint,
+                "ip_address": self.ip_address,
+                "vs": self.index,
+            }
 
     def save(self, update=False):
         if not update:
@@ -452,6 +478,8 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         self.persist = service.get('Persist', None)
         self.persisttimeout = service.get('PersistTimeout', None)
         self.cookie = service.get('Cookie', None)
+        self.extraports = service.get('ExtraPorts', None)
+        self.qos = service.get('QoS', None)
 
     def create_real_server(self, ip, port=80):
         """RealServer factory with pre-configured LoadMaster connection."""
